import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.awt.Color;
/**
 * A piano that can be played with the computer keyboard.
 * 
 * mods: mouse control
 *       octave switch (2 octaves)
 *       keyImage stored in key, set with a boolean parameter true for white, false for black
 * 
 * @author: M. Kolling
 * @modified: Matthew Milan
 * @version: 2.0 
 * @date: 03/06/2012
 */
public class Piano extends World
{
private String[] whiteKeys = {"a","s","d","f","g","h","j","k","l",";","'","\\"};
private String[] blackKeys = {"w","e","t","y","u","o","p","]"};

private String[] whiteNote2 = {"drum0","drum1","drum2","drum3","drum4","drum5","drum6","drum7","drum8","drum9","drum10","drum11"} ;
private String[] blackNote2 = {"bdrum0","bdrum1","bdrum2","bdrum3","bdrum4","bdrum5","bdrum6","bdrum7"};
private String[] whiteNote1 = {"2c","2d","2e","2f","2g","2a","2b","3c","3d","3e","3f","3g"};
private String[] blackNote1 = {"2c#","2d#","2f#","2g#","3a#","3c#","3d#","3f#"};

private String[] activeWhiteNotes = whiteNote1;
private String[] activeBlackNotes = blackNote1;

private Visualizer screen;
    /**
     * Make the piano.
     */
    public Piano() 
    {
        super(800, 340 + 340, 1, false);
        screen = new Visualizer();
        addObject(screen, screen.getImage().getWidth() / 2, screen.getImage().getHeight() / 2);
        makeWhiteKeys();
        makeBlackKeys();
        showMessage();

    }

    /**
     * Pressing 1 or 2 will change the set of sound files to use. in this case, it is a difference
     * of 1 octave
     */
    public void act()
    {
        if (Greenfoot.isKeyDown("1"))
        {
            changeOctave(1);
            octaveIndicator(1);
            showMessage();
        }
        if (Greenfoot.isKeyDown("2"))
        {
            changeOctave(2);
            octaveIndicator(2);
            showMessage();
        }
    }

    /**
     * makes a number of white keys based on the length of whiteKeys array. the total layout
     * of the keys are aligned center-top, no matter what the size of the screen, unless the screen
     * is smaller than the total width of the keys. no scaling is yet impleneted to handle that
     * scenario.
     */

    public void makeWhiteKeys()
    {
        for (int i = 0; i < whiteKeys.length; i ++)
        {
            
            Key newKey = new Key(whiteKeys[i], activeWhiteNotes[i] +  ".wav", false, screen);
            addObject( newKey, 

            (newKey.getImage().getWidth() * i )  // each key's width
               + ( edgeBorder()),

                newKey.getImage().getHeight() / 2 + screen.HEIGHT);

                newKey.setMyColor(new Color(200,0,0,255));
                newKey.setVisualizer(screen);

        }        
    }

    /**
     * makes a number of black keys based on the length of blackKeys array. includes constant values
     * that allow the loop to properly space the black keys in accordance with standard piano layout.
     */
    public void makeBlackKeys()
    {
        int offset = whiteWidth() / 2;
        int spacerCounter = 0;
        final int BLACKKEYSGROUP1 = 2;
        final int BLACKKEYSGROUP2 = 5;
        final int BLACKKEYSGROUP3 = 7;
        final int BLACKKEY_XPOS_ADJUSTMENT = 4;
        
        for (int i = 0; i < blackKeys.length; i ++)
        {
            if (spacerCounter == BLACKKEYSGROUP1 
             || spacerCounter == BLACKKEYSGROUP2 
             || spacerCounter == BLACKKEYSGROUP3)
            {
                offset += whiteWidth();
            }
            Key newKey = new Key(blackKeys[i], activeBlackNotes[i] + ".wav", true,screen);
            addObject( newKey, (whiteWidth() * i ) + offset + edgeBorder(), newKey.getImage().getHeight() / 2  + screen.HEIGHT);
            spacerCounter++;

            newKey.setMyColor(new Color(0,0,Greenfoot.getRandomNumber(200)+50,255));       
            newKey.setVisualizer(screen);
     
        }        
    }

    /**
     * first removes all keys, and then creates a new set of keys based on the button pressed 
     * on the keyboard. key 1 for octave 1, key 2 for octave 2.
     */
    public void changeOctave(int whichOctave)
    {
        removeObjects(getObjects(Key.class));
        if (whichOctave == 1)
        {
            activeWhiteNotes = whiteNote1;
            activeBlackNotes = blackNote1;
            makeWhiteKeys();
            makeBlackKeys();
        }
        if (whichOctave == 2)
        {
            activeWhiteNotes = whiteNote2;
            activeBlackNotes = blackNote2;
            makeWhiteKeys();
            makeBlackKeys();
        }
    }

    /**
     * assists in the placement of keys by returning a value that is as wide as a white key image.
     */
    public int whiteWidth()
    {
        int spacer = new Key("","",false,screen).getImage().getWidth();
        return spacer;
    }
    /**
     * assists in the placement of keys by centering the keyboard based on screen width and 
     * white key width.
     */
    public int edgeBorder()
    {
        return (int) (((float)(getWidth() - ((float)(whiteWidth() * (float)whiteKeys.length))) / 2) + (whiteWidth() / 2) );
    }

    /**
     * writes basic info to the background
     */
    public void showMessage()
    {
        GreenfootImage bg = getBackground();
        bg.setColor(Color.GREEN);
        bg.drawString("INSTRUCTIONS: CLICK RUN. SWITCH OCTAVES WITH 1 AND 2. ", whiteWidth(), 310);
    }

    /**
     * displays which octave is currently selected
     */
    public void octaveIndicator(int whichOctave)
    {
        setBackground("wood.jpg");
        GreenfootImage bg = getBackground();
        bg.setColor(Color.GREEN);
        bg.drawString("Current octave: " + whichOctave, getWidth() - whiteWidth()*4, 320);
    }
}